// -*- mode:java; encoding:utf-8 -*-
// vim:set fileencoding=utf-8:
// https://ateraimemo.com/Swing/KnockoutTextLabel.html

package example;

import java.awt.*;
import java.awt.font.FontRenderContext;
import java.awt.font.GlyphVector;
import java.awt.geom.AffineTransform;
import java.awt.geom.Area;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.InputStream;
import java.util.Optional;
import javax.imageio.ImageIO;
import javax.swing.*;

public final class MainPanel extends JPanel {
  private MainPanel() {
    super(new BorderLayout());
    String path = "example/test.jpg";
    ClassLoader cl = Thread.currentThread().getContextClassLoader();
    Image image = Optional.ofNullable(cl.getResource(path)).map(url -> {
      try (InputStream s = url.openStream()) {
        return ImageIO.read(s);
      } catch (IOException ex) {
        return makeMissingImage();
      }
    }).orElseGet(MainPanel::makeMissingImage);
    JLabel label = new JLabel("ABC") {
      @Override protected void paintComponent(Graphics g) {
        // super.paintComponent(g);
        Graphics2D g2 = (Graphics2D) g.create();
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        g2.drawImage(image, 0, 0, getWidth(), getHeight(), this);
        FontRenderContext frc = g2.getFontRenderContext();
        String text = getText();
        GlyphVector gv = getFont().createGlyphVector(frc, text);
        Rectangle2D b = gv.getVisualBounds();
        double w = getWidth();
        double h = getHeight();
        double cx = w / 2d - b.getCenterX();
        double cy = h / 2d - b.getCenterY();
        AffineTransform toCenterAtf = AffineTransform.getTranslateInstance(cx, cy);
        Shape s = toCenterAtf.createTransformedShape(gv.getOutline());
        Area bg = new Area(new Rectangle2D.Double(0d, 0d, w, h));
        bg.subtract(new Area(s));
        g2.setColor(getBackground());
        g2.fill(bg);
        g2.dispose();
      }
    };
    label.setBackground(new Color(0xE6_00_00_32, true));
    label.setFont(new Font(Font.SERIF, Font.BOLD, 140));

    JCheckBox check = new JCheckBox("hasAlpha", true);
    check.addActionListener(e -> {
      Color color = new Color(0xE6_00_00_32, ((JCheckBox) e.getSource()).isSelected());
      label.setBackground(color);
    });

    add(label);
    add(check, BorderLayout.SOUTH);
    setPreferredSize(new Dimension(320, 240));
  }

  private static Image makeMissingImage() {
    Icon missingIcon = UIManager.getIcon("OptionPane.errorIcon");
    int w = missingIcon.getIconWidth();
    int h = missingIcon.getIconHeight();
    BufferedImage bi = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);
    Graphics2D g2 = bi.createGraphics();
    missingIcon.paintIcon(null, g2, 0, 0);
    g2.dispose();
    return bi;
  }

  public static void main(String[] args) {
    EventQueue.invokeLater(MainPanel::createAndShowGui);
  }

  private static void createAndShowGui() {
    try {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    } catch (UnsupportedLookAndFeelException ignored) {
      Toolkit.getDefaultToolkit().beep();
    } catch (ClassNotFoundException | InstantiationException | IllegalAccessException ex) {
      ex.printStackTrace();
      return;
    }
    JFrame frame = new JFrame("KnockoutTextLabel");
    frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
    frame.getContentPane().add(new MainPanel());
    frame.pack();
    frame.setLocationRelativeTo(null);
    frame.setVisible(true);
  }
}
